/*
  (c) 2006,2007 Wojciech M. Zabolotny <wzab@ise.pw.edu.pl>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "common_types.h"
#include "dso2100.h"
#include <pic18fregs.h>
#include <delay.h>

/*

  WZab remarks.
  It seems, that the order of setting of the LPT lines is essential.
  My USB interface didn't work, until the INIT was the last line set before the STROBE!

 */

/* The variable dummy_var_for_bug_protection has been introduced to
  workaround the bug in the SDCC: 
  http://sourceforge.net/tracker/index.php?func=detail&aid=1818857&group_id=599&atid=100599
  When the bug gets fixed, you can get rid of this variable */
uchar dummy_var_for_bug_protection=0;
//#pragma udata usb_buf osc_buffer
//uchar osc_buffer[BUFFERLEN];
dso_params dso_config;

uchar dso_verify_connection(void)
{
  uchar res;
  DTAOUT=0xff;
  data_to_scope;
  STROBE=1; AUTO=1; SLCTIN=1; INIT=1; //Added WZab
  delay100tcy(10); //Added WZab
  STROBE=1; AUTO=1; SLCTIN=1; INIT=0;
  delay100tcy(10);
  data_from_scope;
  STROBE=0;
  delay100tcy(10);
  res=DTAIN;
  STROBE=1;
  //return (res==0x55) ? 1 : 0;
  return res;
}

uchar dso_read_status(void)
{
  uchar res;
  STROBE=1; AUTO=0; SLCTIN=1; INIT=0;
  delay10tcy(1);
  STROBE=0;
  delay10tcy(40);
  res=DTAIN;
  STROBE=1; 
  delay10tcy(10); //I don't know why, but the original code waits in this state...
  AUTO=1; SLCTIN=1; INIT=1; //Init moved to the end?
  return res;
}

uchar dso_verify_status(uchar req_stat)
{
  return (dso_read_status() == req_stat) ? 0 : 1;
}
void dso_send_command(uchar cmd)
{
  STROBE=1; AUTO=0; SLCTIN=1; INIT=1;
  //Switch on the data bus drivers
  delay10tcy(10);
  data_to_scope;
  DTAOUT=cmd;
  delay10tcy(10);
  STROBE=0;
  delay10tcy(10);
  STROBE=1; 
  //Switch off the data bus drivers
  data_from_scope;
  delay10tcy(10);
  AUTO=1;
  SLCTIN=1;
  INIT=1;
}

//The load data command assumes, that the data bus drivers in the PIC
//are permanently off (it is reasonable, as the transfer from DSO
//to PC is used much more often than from PC to DSO
uchar dso_load_data(uchar chan)
{
  uchar res;
  AUTO=chan ? 1 : 0; // Changed the order AUTO before SLCTIN 
  SLCTIN=0; // Only after that, the readout started to work correctly...
  INIT=0;
  data_from_scope;
  STROBE=1;
  delay10tcy(1);
  STROBE=0;
  delay10tcy(40);
  res=DTAIN;
  STROBE=1;
  return res;
}

uchar dso_transfer_parameters(void)
{
  uint res;
  if(dso_verify_status(0x0f)) {
    while (1) {
      if(!dso_verify_status(0x0f)) break;
      dso_send_command(0xff);
      if(!dso_verify_status(0x0f)) break;
      dso_send_command(0x04);
    }
  }
  res=5;
  if(dso_verify_status(0x0f)) goto err_dtp;
  dso_send_command(dso_config.ack_chn);
  res++;
  if(dso_verify_status(0x0f)) goto err_dtp;
  dso_send_command(dso_config.ack_chn);
  res++;
  if(dso_verify_status(0x05)) goto err_dtp;
  dso_send_command(dso_config.smp_rate);
  res++;
  if(dso_verify_status(0x05)) goto err_dtp;
  dso_send_command(dso_config.smp_rate);
  res++;
  if(dso_verify_status(0x08)) goto err_dtp;
  dso_send_command(dso_config.rd2_step);
  res++;
  if(dso_verify_status(0x08)) goto err_dtp;
  dso_send_command(dso_config.rd2_step);
  res++;
  if(dso_verify_status(0x09)) goto err_dtp;
  dso_send_command(dso_config.rd1_step);
  res++;
  if(dso_verify_status(0x09)) goto err_dtp;
  dso_send_command(dso_config.rd1_step);
  res++;
  if(dso_verify_status(0x06)) goto err_dtp;
  dso_send_command(dso_config.ch1_trig);
  res++;
  if(dso_verify_status(0x06)) goto err_dtp;
  dso_send_command(dso_config.ch1_trig);
  res++;
  if(dso_verify_status(0x07)) goto err_dtp;
  dso_send_command(dso_config.ch2_trig);
  res++;
  if(dso_verify_status(0x07)) goto err_dtp;
  dso_send_command(dso_config.ch2_trig);
  res++;
  if(dso_verify_status(0x0a)) goto err_dtp;
  dso_send_command(dso_config.rd2_shift);
  res++;
  if(dso_verify_status(0x0a)) goto err_dtp;
  dso_send_command(dso_config.rd2_shift);
  res++;
  if(dso_verify_status(0x0b)) goto err_dtp;
  dso_send_command(dso_config.trig_slope);
  res++;
  if(dso_verify_status(0x0b)) goto err_dtp;
  dso_send_command(dso_config.trig_slope);
  res++;
  if(dso_verify_status(0x0c)) goto err_dtp;
  dso_send_command(dso_config.ch1_v_offset);
  res++;
  if(dso_verify_status(0x0c)) goto err_dtp;
  dso_send_command(dso_config.ch1_v_offset);
  res++;
  if(dso_verify_status(0x0d)) goto err_dtp;
  dso_send_command(dso_config.ch2_v_offset);
  res++;
  if(dso_verify_status(0x0d)) goto err_dtp;
  dso_send_command(dso_config.ch2_v_offset);
  res++;
  if(dso_verify_status(0x0e)) goto err_dtp;
  dso_send_command(dso_config.trig_level);
  res++;
  if(dso_verify_status(0x0e)) goto err_dtp;
  dso_send_command(dso_config.trig_level);
  res++;
  if(dso_verify_status(0x10)) goto err_dtp;
  dso_send_command(dso_config.ch1_offset);
  res++;
  if(dso_verify_status(0x10)) goto err_dtp;
  dso_send_command(dso_config.ch1_offset);
  res++;
  if(dso_verify_status(0x11)) goto err_dtp;
  dso_send_command(dso_config.ch1_gain);
  res++;
  if(dso_verify_status(0x11)) goto err_dtp;
  dso_send_command(dso_config.ch1_gain);
  res++;
  if(dso_verify_status(0x12)) goto err_dtp;
  dso_send_command(dso_config.ch2_offset);
  res++;
  if(dso_verify_status(0x12)) goto err_dtp;
  dso_send_command(dso_config.ch2_offset);
  res++;
  if(dso_verify_status(0x13)) goto err_dtp;
  dso_send_command(dso_config.ch2_gain);
  res++;
  if(dso_verify_status(0x13)) goto err_dtp;
  dso_send_command(dso_config.ch2_gain);
  res++;
  if(dso_verify_status(0x1)) goto err_dtp;
  //Parameters sent correctly, semd the acknowledge
  return 0;
 err_dtp:
  //Error during the sending of the parameters
  //Bring the scope to the stable state
  //Signal the error
  return res;
}

/*
  The data download routines...
  We need three kinds of the data download routines...
  1) Download only the rd1
  2) Download both rd1 and rd2
  3) Download all the data: rd1, rd2 and all 5000 samples...
 */

uchar dso_start_acq(uchar * p)
{
  uint timeout;
  uint res;
  timeout =*((uint *) (p+1));
  if (((*p) & (0x01 << 5)) == 0) {
    // This is really start of acquisition command!
    dso_config.ack_chn = (*p) & 0x03; 
    res=dso_transfer_parameters();
    if(res)
      return res; // error
    dso_send_command(0x02);
    // Now we are waiting until the data are acquired
    while(timeout) {
      uchar tmp;
      tmp = dso_read_status();
      if(tmp == 0x21) {
	//The data are successfully acquired
	return 0;
      }
      if(tmp != 0x00) {
	//This shouldn't happen!!!
	return 2;
      }
    }
    //Timeout occured!
    return 1;
  }
  
}

void dso_pic_init(void)
{
  PORTE=0;
  ADCON1=0x0f;
  CMCON=0x07;
  PORTD=0;
  PORTE=0x87; //RDPU=1
  PORTB=0x10;
  data_from_scope;
  TRISE=0; 
  TRISB=0xff-0x10; //RB4 as output
}
