/*-------------------------------------------------------------------------
  ep1.c - USB endpoint 1 callbacks

  (c) 2006 Pierre Gaufillet <pierre.gaufillet@magic.fr> 
  (c) 2006,2007 Modified by Wojciech M. Zabolotny <wzab@ise.pw.edu.pl>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

  Seriously modified by Wojciech M. Zabolotny <wzab@ise.pw.edu.pl>
  The USB bootloader functionality removed
  Instead the JTAG controller functionality introduced
  -------------------------------------------------------------------------*/

/* $Id: ep1.c,v 1.5 2006/04/18 19:44:32 gaufille Exp $ */

#include "ep1.h"
#include "ep2.h"
#include "usb.h"
#include "debug.h"
#include "flash.h"
#include "dso2100.h"
#include <pic18fregs.h>
#include <delay.h>
#include <string.h>

#pragma udata usb_buf ep1_OutBuffer
volatile uchar ep1_OutBuffer[EP1_BUFFER_SIZE];
extern volatile uchar ep2_InBuffer[EP2_BUFFER_SIZE];

#pragma udata usb_buf osc_buffer
uchar osc_buffer[BUFFERLEN];

uchar code dev_id_message[]="USB DSO-2100 GPL Interface 1.1";
//uchar data result=0xa5;

void ep1_init(void)
{
  debug("ep1_init\n");
  EP_OUT_BD(1).Cnt = EP1_BUFFER_SIZE;
  EP_OUT_BD(1).ADR = (uchar __data *)&ep1_OutBuffer;
  EP_OUT_BD(1).Stat.uc = BDS_USIE | BDS_DAT0 | BDS_DTSEN;
  UEP1 = EPHSHK_EN | EPOUTEN_EN | EPCONDIS_EN;       // Init EP1 as an OUT EP
}

void ep1_out(void)
{
  debug("ep1_out\n");
  if(EP_OUT_BD(1).Cnt >= 1)
    {
      short len;
      //First decode commands encoded on four most significant bits 
      switch(ep1_OutBuffer[0])
        {
	case DSO_GET_INFO:
	  ep2_num_bytes_to_send=sizeof(dev_id_message);
	  ep2_source_data=dev_id_message;
	  prepare_ep2_in();
	  break;
	case DSO_SEND_CMD:
	  if(EP_OUT_BD(1).Cnt != 2) {
	    //It is not correct buffer size!!!
	    //Do something
	    goto error;
	  };
	  dso_send_command(ep1_OutBuffer[1]);
	  ep2_num_bytes_to_send = 1;
	  ep2_InBuffer[0]='A';//DSO_ACK;
	  prepare_ep2_in_no_copy();
	  break;
	case DSO_GET_STATUS:
	  ep2_num_bytes_to_send = 2;
	  ep2_InBuffer[0]=dso_read_status();
	  ep2_InBuffer[1]='B';//DSO_ACK;
	  prepare_ep2_in_no_copy();
	  /*result=0xd0;
	    ep2_source_data=&result;
	    prepare_ep2_in();*/
	  break;
	case DSO_SEND_CMD_GET_STATUS:
	  if(EP_OUT_BD(1).Cnt != 2) {
	    //It is not correct buffer size!!!
	    //Do something
	    goto error;
	  };
	  dso_send_command(ep1_OutBuffer[1]);
	  ep2_num_bytes_to_send = 2;
	  ep2_InBuffer[0]=dso_read_status();
	  ep2_InBuffer[1]='C';//DSO_ACK;
	  prepare_ep2_in_no_copy();
	  /*result=0xd0;
	    ep2_source_data=&result;
	    prepare_ep2_in();*/
	  break;
	case DSO_SET_PARAMS:
	  // We send all parameters to the interface
	  if(EP_OUT_BD(1).Cnt != 1+sizeof(dso_config)) {
	    //It is not correct buffer size!!!
	    //Do something
	    goto error;
	  };
	  memcpy((void *) &dso_config,(void *)&ep1_OutBuffer[1],sizeof(dso_config));
	  ep2_num_bytes_to_send = 1;
	  ep2_InBuffer[0]='D';//DSO_ACK;
	  prepare_ep2_in_no_copy();
	  //result=0xc0 | jtag_set_pins(ep1_OutBuffer[0]);
	  //ep2_source_data=&result;
	  //prepare_ep2_in();
	  break;
	case DSO_SET_PARAM:
	  //We set the single parameter at chosen position
	  if(EP_OUT_BD(1).Cnt != 3) {
	    //It is not correct buffer size!!!
	    //Do something
	    goto error;
	  };
	  if(ep1_OutBuffer[1]>=sizeof(dso_config)) {
	    // Someone tried to write outside the dso_config...
	    goto error;
	  }
	  *((uchar *)&dso_config + ep1_OutBuffer[1]) = ep1_OutBuffer[2];
	  ep2_num_bytes_to_send = 1;
	  ep2_InBuffer[0]='E';//DSO_ACK;
	  prepare_ep2_in_no_copy();
	  break;
	case DSO_START_ACQ:
	  //We start the acquisition in a chosen channel
	  //First byte: bit 0 - the channel, bit 7 - the auto mode, 
	  // bit 6 - no acquisition is started, we only wait for the end
	  // of the acquisition
	  //bits 5-1 - currently unused
	  //The second byte encodes the LSB of the timeout
	  //The third byte: MSB of the timeout
	  if(EP_OUT_BD(1).Cnt != 4) {
	    //It is not correct buffer size!!!
	    //Do something
	    goto error;
	  };
	  ep2_num_bytes_to_send = 1;
	  ep2_InBuffer[0]=dso_start_acq(&ep1_OutBuffer[1]);
	  prepare_ep2_in_no_copy();
	  break;
	case DSO_GET_DATA:
	  //We get specified amount of data from the chosen channel
	  if(EP_OUT_BD(1).Cnt != 3) {
	    //It is not correct buffer size!!!
	    //Do something
	    goto error;
	  };
	  if(ep1_OutBuffer[1]>BUFFERLEN) {
	    // We can only transfer the data in the chunks of 250 bytes
	    // (we can modify this setting later)
	    goto error;
	    }
	  //Read the data from the DSO, and put them into the buffer
	  {
	    int i;
	    for(i=0;i<ep1_OutBuffer[1];i++) {
	      osc_buffer[i]=dso_load_data(ep1_OutBuffer[2]);
	    }
	  }
	  ep2_source_data=osc_buffer;
	  ep2_num_bytes_to_send = ep1_OutBuffer[1];
	  prepare_ep2_in();
	  break;	  
	case DSO_VERIFY_HW:
	  // Check if the DSO2100 is present
	  ep2_num_bytes_to_send = 2;
	  ep2_InBuffer[0]=dso_verify_connection();
	  ep2_InBuffer[1]='F';//DSO_ACK;
	  prepare_ep2_in_no_copy();
	  break;
	default:
	  debug("unknown command\n");
	  // Raise an error
	  EP_OUT_BD(1).Cnt = EP1_BUFFER_SIZE;
	  EP_OUT_BD(1).ADR = (uchar __data *)&ep1_OutBuffer;
	  EP_OUT_BD(1).Stat.uc = BDS_USIE | BDS_BSTALL;
	  break;
        }
      EP_OUT_BD(1).Cnt = EP1_BUFFER_SIZE;
      if(EP_OUT_BD(1).Stat.DTS == 0)
        {
	  EP_OUT_BD(1).Stat.uc = BDS_USIE | BDS_DAT1 | BDS_DTSEN;
        }
      else
        {
	  EP_OUT_BD(1).Stat.uc = BDS_USIE | BDS_DAT0 | BDS_DTSEN;
        }
    }
  return;
 error:
  // Raise an error
  {
    EP_OUT_BD(1).Cnt = EP1_BUFFER_SIZE;
    EP_OUT_BD(1).ADR = (uchar __data *)&ep1_OutBuffer;
    EP_OUT_BD(1).Stat.uc = BDS_USIE | BDS_BSTALL;
  }
}
